@@ -1,7 +1,7 @@
 from computemate.core.systems import *
 from computemate.ui.text_area import getTextArea
 from computemate.ui.info import get_banner
-from computemate import config, DIALOGS, COMPUTEMATE_VERSION, AGENTMAKE_CONFIG, COMPUTEMATE_PACKAGE_PATH, COMPUTEMATE_USER_DIR, COMPUTEMATEDATA, fix_string, write_user_config, edit_mcp_config_file, get_mcp_config_file
+from computemate import config, DIALOGS, COMPUTEMATE_VERSION, AGENTMAKE_CONFIG, COMPUTEMATE_PACKAGE_PATH, COMPUTEMATE_USER_DIR, COMPUTEMATEDATA, fix_string, write_user_config, edit_mcp_config_file, get_mcp_config_file, run_system_command, list_dir_content
 from pathlib import Path
 import urllib.parse
 import asyncio, re, os, subprocess, click, gdown, pprint, argparse, json, zipfile, warnings, sys
@@ -9,6 +9,7 @@ from copy import deepcopy
 from alive_progress import alive_bar
 from fastmcp import Client
 from fastmcp.client.transports import StreamableHttpTransport
+from agentmake.utils.system import getDeviceInfo
 from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_file, edit_configurations, readTextFile, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_AI_BACKEND, DEFAULT_TEXT_EDITOR
 from agentmake.utils.files import searchFolder, isExistingPath
 from agentmake.etextedit import launch_async
@@ -212,19 +213,6 @@ def backup_conversation(messages, master_plan, console=None, storage_path=None):
             info = f"Conversation saved to: {storage_path}\nReport saved to: {html_file}"
             display_info(console, info)
 
-def run_system_command(cmd: str):
-    cmd += " && cd" if USER_OS == "Windows" else " && pwd"
-    text_output = subprocess.run(
-        cmd,
-        shell=True,
-        capture_output=True,
-        text=True,
-    ).stdout.strip()
-    lines = text_output.split("\n")
-    if len(lines) == 1:
-        return "Done!", lines[0]
-    return "\n".join(lines[:-1]), lines[-1]
-
 async def main_async():
 
     # get mcp sefver configurations
@@ -235,9 +223,9 @@ async def main_async():
     computemate_mcp_server = user_mcp_server if os.path.isfile(user_mcp_server) else builtin_mcp_server
     config_mcp["computemate"] = {"command": "python", "args": [computemate_mcp_server]}
     # format config dict
-    config_mcp = {"mcpServers": config_mcp}
+    config_mcp_client = {"mcpServers": config_mcp}
     # set client
-    client = Client(config_mcp)
+    client = Client(config_mcp_client)
 
     APP_START = True
     DEFAULT_SYSTEM = "You are ComputeMate AI, an autonomous agent designed to assist users with using computers."
@@ -320,6 +308,9 @@ async def main_async():
                     if latest_version and latest_version > version.parse(COMPUTEMATE_VERSION):
                         info = f"A new version of ComputeMate AI is available: {latest_version} (you are using {COMPUTEMATE_VERSION}).\nTo upgrade, close `ComputeMate AI` first and run `pip install --upgrade computemate`."
                         display_info(console, info)
+                    # list current directory content
+                    cwd = os.getcwd()
+                    display_info(console, list_dir_content(cwd), title=cwd)
                     # check connection
                     if not config.skip_connection_check:
                         try:
@@ -334,7 +325,7 @@ async def main_async():
             # Original user request
             # note: `python3 -m rich.emoji` for checking emoji
             console.print("Enter your request :smiley: :" if len(messages) == len(DEFAULT_MESSAGES) else "Enter a follow-up request :flexed_biceps: :")
-            input_suggestions = list(config.action_list.keys())+["@ ", "@@ "]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+[f"//{r}" for r in resources.keys()]+template_list+resource_suggestions
+            input_suggestions = list(config.action_list.keys())+["@ ", "@@ ", "!", "!!"]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+[f"//{r}" for r in resources.keys()]+template_list+resource_suggestions+sorted(os.listdir("."))+[f"??{i}?? " for i in sorted(os.listdir("."))]+config.custom_input_suggestions
             if args.default:
                 user_request = " ".join(args.default).strip()
                 args.default = None # reset to avoid repeated use
@@ -344,26 +335,47 @@ async def main_async():
                 master_plan = ""
             # open a text file as a prompt
             check_path = isExistingPath(user_request)
-            if check_path:
+            if check_path and not user_request == ".":
                 config.current_prompt = readTextFile(check_path)
                 continue
             # luanch action menu
-            if user_request == ".":
+            if not user_request:
+                continue
+            elif user_request == ".":
                 select = await DIALOGS.getValidOptions(options=config.action_list.keys(), descriptions=[i.capitalize() for i in config.action_list.values()], title="Action Menu", text="Select an action:")
                 user_request = select if select else ""
             elif user_request.startswith("!"):
                 pre_cwd = os.getcwd()
-                cmd = user_request[1:].strip()
+                if user_request.startswith("!!"):
+                    record_cmd = False
+                    cmd = user_request[2:].strip()
+                else:
+                    record_cmd = True
+                    cmd = user_request[1:].strip()
                 if not cmd:
                     cmd = "cd" if USER_OS == "Windows" else "pwd"
+                if not record_cmd:
+                    os.system(cmd)
+                    print()
+                    continue
                 cmd_output, cwd = run_system_command(cmd)
-                display_info(console, cmd_output)
+                display_info(console, Markdown(f"```\n{cmd_output}\n```"))
+                messages += [
+                    {"role": "user", "content": f"Run system command:\n\n```\n{cmd}\n```"},
+                    {"role": "assistant", "content": f"```output\n{cmd_output}\n```"},
+                ]
                 if (not pre_cwd == cwd) and os.path.isdir(cwd):
                     os.chdir(cwd)
-                    display_info(console, cwd, title="Current Directory")
-                continue
-            if not user_request:
+                    display_info(console, list_dir_content(cwd), title=cwd)
                 continue
+            elif re.search(r"\?\?(.*?)\?\? ", user_request+" "):
+                fileList = [os.path.abspath(i) for i in re.findall(r"\?\?(.*?)\?\? ", user_request+" ")]
+                new_user_request = re.sub(r"\?\?(.*?)\?\? ", "", user_request+" ").strip()
+                new_user_request = f'''# File List\n\n{fileList}\n\n# User Request\n\n{new_user_request}'''
+                if config.agent_mode is None and not user_request.startswith("@"):
+                    user_request = ("@computemate_ask_files " if len(config_mcp) > 1 else "@ask_files ") + "\n\n" + new_user_request
+                else:
+                    user_request = new_user_request
             if user_request == ".ideas":
                 # Generate ideas for `prompts to try`
                 ideas = ""
@@ -407,10 +419,9 @@ async def main_async():
                 try:
                     template_name, template_args = user_request[2:].split("/", 1)
                     if template_name in ("computemate_content", "content"):
-                        if not template_args:
-                            template_args = "."
-                        else:
-                            template_args = template_args.replace("/", "%2F")
+                        if not template_args or template_args == ".":
+                            template_args = os.getcwd()
+                        template_args = template_args.replace("/", "%2F")
                     uri = re.sub("{.*?$", "", templates[template_name][1])+template_args
                     resource_content = await client.read_resource(uri)
                     resource_content = resource_content[0].text
@@ -434,7 +445,6 @@ async def main_async():
             if user_request.startswith(".open") or user_request.startswith(".import") or user_request.startswith(".reload"):
                 cwd = os.getcwd()
             if user_request == ".open":
-                os.chdir(COMPUTEMATE_USER_DIR)
                 open_item = await DIALOGS.getInputDialog(title="Open", text="Enter a file or folder path:", suggestions=PathCompleter())
                 if not open_item:
                     open_item = os.getcwd()
@@ -549,6 +559,9 @@ Viist https://github.com/eliranwong/computemate
 - `Esc+G`: generate ideas for prompts to try
 - `Ctrl+P`: toggle auto prompt engineering
 - `Esc+P`: improve prompt content
+- `Esc+T`: toggle auto tool selection in chat mode
+- `Ctrl+C`: change directory
+- `Esc+C`: show current directory content
 - `Ctrl+D`: delete
 - `Ctrl+H`: backspace
 - `Ctrl+W`: delete previous word
@@ -695,16 +708,51 @@ Viist https://github.com/eliranwong/computemate
                         except:
                             info = "Invalid input."
                             display_info(console, info)
-                elif user_request == ".promptengineer":
+                elif user_request == ".content":
+                    cwd = os.getcwd()
+                    display_info(console, list_dir_content(cwd), title=cwd)
+                elif user_request == ".autoprompt":
                     config.prompt_engineering = not config.prompt_engineering
                     write_user_config()
                     info = f"Prompt Engineering {'Enabled' if config.prompt_engineering else 'Disabled'}!"
                     display_info(console, info)
-                elif user_request == ".autosuggestions":
+                elif user_request == ".autosuggest":
                     config.auto_suggestions = not config.auto_suggestions
                     write_user_config()
                     info = f"Auto Input Suggestions {'Enabled' if config.auto_suggestions else 'Disabled'}!"
                     display_info(console, info)
+                elif user_request == ".autotool":
+                    config.auto_tool_selection = not config.auto_tool_selection
+                    write_user_config()
+                    info = f"Auto Tool Selection in Chat Mode {'Enabled' if config.auto_tool_selection else 'Disabled'}!"
+                    display_info(console, info)
+                elif user_request == ".autocorrect":
+                    config.auto_code_correction = not config.auto_code_correction
+                    write_user_config()
+                    info = f"Auto Code Correction {'Enabled' if config.auto_code_correction else 'Disabled'}!"
+                    display_info(console, info)
+                elif user_request == ".directory":
+                    directory = os.getcwd()
+                    while directory and not directory == ".":
+                        options = [".", ".."]+[i for i in os.listdir(directory) if os.path.isdir(os.path.join(directory, i))]
+                        select = await DIALOGS.getValidOptions(
+                            options=options,
+                            title="Change Directory",
+                            text="Select a directory:"
+                        )
+                        if select:
+                            if select == ".":
+                                break
+                            elif select == "..":
+                                directory = os.path.dirname(directory)
+                            else:
+                                directory = os.path.join(directory, select)
+                        else:
+                            break
+                    if select:
+                        os.chdir(directory)
+                        cwd = os.getcwd()
+                        display_info(console, list_dir_content(cwd), title=cwd)
                 elif user_request == ".lite":
                     config.lite = not config.lite
                     write_user_config()
@@ -755,6 +803,9 @@ Viist https://github.com/eliranwong/computemate
                     messages = deepcopy(DEFAULT_MESSAGES)
                     console.clear()
                     console.print(get_banner(COMPUTEMATE_VERSION))
+                    # show current directory content
+                    cwd = os.getcwd()
+                    display_info(console, list_dir_content(cwd), title=cwd)
                 continue
 
             # Check if a single tool is specified
@@ -764,6 +815,10 @@ Viist https://github.com/eliranwong/computemate
             # Tool selection systemm message
             system_tool_selection = get_system_tool_selection(available_tools, tool_descriptions)
 
+            # auto tool selection in chat mode
+            if config.agent_mode is None and config.auto_tool_selection and not user_request.startswith("@"):
+                user_request = f"@ {user_request}"
+            
             if user_request.startswith("@ "):
                 user_request = user_request[2:].strip()
                 # Single Tool Suggestion
@@ -785,7 +840,9 @@ Viist https://github.com/eliranwong/computemate
                 # Single Tool Selection
                 if config.agent_mode:
                     this_tool = suggested_tools[0] if suggested_tools else "get_direct_text_response"
-                else: # `partner` mode when config.agent_mode is set to False
+                elif config.agent_mode is None and suggested_tools[0] == "get_direct_text_response":
+                    this_tool = "get_direct_text_response"
+                else: # `partner` or `chat`mode when config.agent_mode is set to False or None
                     this_tool = await DIALOGS.getValidOptions(options=suggested_tools if suggested_tools else available_tools, title="Suggested Tools", text="Select a tool:")
                     if not this_tool:
                         this_tool = "get_direct_text_response"
@@ -849,22 +906,45 @@ Viist https://github.com/eliranwong/computemate
                 nonlocal messages
                 tool_instruction = fix_string(tool_instruction)
                 messages[-1]["content"] = fix_string(messages[-1]["content"])
+                request_dict = [{"role": "system", "content": DEFAULT_SYSTEM}]+messages[len(messages)-2:] if config.lite else deepcopy(messages)
                 if tool == "get_direct_text_response":
                     messages = agentmake(messages, system="auto", **AGENTMAKE_CONFIG)
                 else:
                     try:
                         tool_schema = tools_schema[tool]
                         tool_properties = tool_schema["parameters"]["properties"]
-                        if len(tool_properties) == 1 and "request" in tool_properties: # AgentMake MCP Servers or alike
-                            if "items" in tool_properties["request"]: # requires a dictionary instead of a string
-                                request_dict = [{"role": "system", "content": DEFAULT_SYSTEM}]+messages[len(messages)-2:] if config.lite else deepcopy(messages)
-                                tool_result = await client.call_tool(tool, {"request": request_dict}, timeout=config.mcp_timeout)
-                            else:
-                                tool_result = await client.call_tool(tool, {"request": tool_instruction}, timeout=config.mcp_timeout)
+                        if tool in ("computemate_execute_task", "execute_task"):
+                            tool_instruction = "# Instruction\n\n"+tool_instruction+"\n\n# Supplementary Device Information\n\n"+getDeviceInfo()
+                            tool_result = agentmake(tool_instruction, **{'tool': 'magic' if config.auto_code_correction else 'execute_task'}, **AGENTMAKE_CONFIG)[-1].get("content") if messages and "content" in messages[-1] else "Error!"
+                        elif tool in ("computemate_email_outlook", "email_outlook", "computemate_email_gmail", "email_gmail"):
+                            tool_result = agentmake(tool_instruction, **{'tool': 'email/outlook' if 'outlook' in tool else 'email/gmail'}, **AGENTMAKE_CONFIG)[-1].get("content") if messages and "content" in messages[-1] else "Error!"
+                        elif tool in ("computemate_calendar_outlook", "calendar_outlook", "computemate_calendar_google", "calendar_google"):
+                            tool_result = agentmake(tool_instruction, **{'input_content_plugin': 'convert_relative_datetime', 'tool': 'calendar/outlook' if 'outlook' in tool else 'calendar/google'}, **AGENTMAKE_CONFIG)[-1].get("content") if messages and "content" in messages[-1] else "Error!"
+                        elif tool in ("computemate_teamwork", "teamwork"):
+                            this_AGENTMAKE_CONFIG = deepcopy(AGENTMAKE_CONFIG)
+                            this_AGENTMAKE_CONFIG["print_on_terminal"] = True
+                            this_AGENTMAKE_CONFIG["word_wrap"] = True
+                            tool_result = agentmake(request_dict, **{'agent': 'teamwork'}, **this_AGENTMAKE_CONFIG)[-1].get("content") if messages and "content" in messages[-1] else "Error!"
+                        elif tool in ("computemate_reflection_agent", "reflection_agent"):
+                            this_AGENTMAKE_CONFIG = deepcopy(AGENTMAKE_CONFIG)
+                            this_AGENTMAKE_CONFIG["print_on_terminal"] = True
+                            this_AGENTMAKE_CONFIG["word_wrap"] = True
+                            tool_result = agentmake(request_dict, **{'agent': 'deep_reflection'}, **this_AGENTMAKE_CONFIG)[-1].get("content") if messages and "content" in messages[-1] else "Error!"
+                        elif tool in ("computemate_reasoning_agent", "reasoning_agent"):
+                            this_AGENTMAKE_CONFIG = deepcopy(AGENTMAKE_CONFIG)
+                            this_AGENTMAKE_CONFIG["print_on_terminal"] = True
+                            this_AGENTMAKE_CONFIG["word_wrap"] = True
+                            tool_result = agentmake(request_dict, **{'agent': 'reasoning'}, **this_AGENTMAKE_CONFIG)[-1].get("content") if messages and "content" in messages[-1] else "Error!"
                         else:
-                            structured_output = getDictionaryOutput(messages=messages, schema=tool_schema, backend=config.backend)
-                            tool_result = await client.call_tool(tool, structured_output, timeout=config.mcp_timeout)
-                        tool_result = tool_result.content[0].text
+                            if len(tool_properties) == 1 and "request" in tool_properties: # AgentMake MCP Servers or alike
+                                if "items" in tool_properties["request"]: # requires a dictionary instead of a string
+                                    tool_result = await client.call_tool(tool, {"request": request_dict}, timeout=config.mcp_timeout)
+                                else:
+                                    tool_result = await client.call_tool(tool, {"request": tool_instruction}, timeout=config.mcp_timeout)
+                            else:
+                                structured_output = getDictionaryOutput(messages=messages, schema=tool_schema, backend=config.backend)
+                                tool_result = await client.call_tool(tool, structured_output, timeout=config.mcp_timeout)
+                            tool_result = tool_result.content[0].text
                         messages[-1]["content"] += f"\n\n[Using tool `{tool}`]"
                         messages.append({"role": "assistant", "content": tool_result if tool_result.strip() else "Tool error!"})
                     except Exception as e:
@@ -1011,7 +1091,7 @@ Available tools are: {available_tools}.
                     else:
                         next_tool_description = tools.get(next_tool, "No description available.")
                         system_tool_instruction = get_system_tool_instruction(next_tool, next_tool_description)
-                        next_step = agentmake(next_suggestion, system=system_tool_instruction, **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
+                        next_step = agentmake([{"role": "system", "content": system_tool_instruction}]+messages[len(DEFAULT_MESSAGES):], follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
                 await thinking(get_next_step, "Crafting the next instruction ...")
                 # partner mode
                 if config.agent_mode == False:
